/*
 * nrf_task.c
 *
 *  Created on: December 23, 2019
 *      Author: lidq
 */

#include <stdint.h>
#include <stdio.h>
//
#include <crc.h>
#include <flash.h>
#include <nrf2401.h>
#include <nrf2401_task.h>

#define MAGIC_NUM (0x1F28E9C4)

#define CTL_OFFSET_MIN (-300)
#define CTL_OFFSET_MAX (300)

extern uint16_t ctl_pwm[ADC_CHS];
extern uint16_t ctl_btn[BTN_CHS];

uint16_t pwm[8] = { 0 };
int16_t ctl_offset[ADC_CHS] = { 0 };

int tx_send = 0;

void ctl_offset_limit(int16_t* value)
{
	if (*value < CTL_OFFSET_MIN)
	{
		*value = CTL_OFFSET_MIN;
	}
	if (*value > CTL_OFFSET_MAX)
	{
		*value = CTL_OFFSET_MAX;
	}
}

void ctl_pwm_limit(uint16_t* value)
{
	if (*value < CTL_PWM_MIN)
	{
		*value = CTL_PWM_MIN;
	}
	if (*value > CTL_PWM_MAX)
	{
		*value = CTL_PWM_MAX;
	}
}

void ctl_set_offset(void)
{
	// CH0
	if (ctl_btn[2] == 1)
	{
		ctl_offset[0]++;
		ctl_offset_limit(&ctl_offset[0]);
	}
	if (ctl_btn[3] == 1)
	{
		ctl_offset[0]--;
		ctl_offset_limit(&ctl_offset[0]);
	}
	// CH1
	if (ctl_btn[0] == 1)
	{
		ctl_offset[1]--;
		ctl_offset_limit(&ctl_offset[1]);
	}
	if (ctl_btn[1] == 1)
	{
		ctl_offset[1]++;
		ctl_offset_limit(&ctl_offset[1]);
	}
	// CH2
	if (ctl_btn[6] == 1)
	{
		ctl_offset[2]++;
		ctl_offset_limit(&ctl_offset[2]);
	}
	if (ctl_btn[7] == 1)
	{
		ctl_offset[2]--;
		ctl_offset_limit(&ctl_offset[2]);
	}
	// CH3
	if (ctl_btn[4] == 1)
	{
		ctl_offset[3]++;
		ctl_offset_limit(&ctl_offset[3]);
	}
	if (ctl_btn[5] == 1)
	{
		ctl_offset[3]--;
		ctl_offset_limit(&ctl_offset[3]);
	}

	for (int i = 0; i < 4; i++)
	{
		pwm[i] = ctl_pwm[i] + ctl_offset[i];
		ctl_pwm_limit(&pwm[i]);
	}
}

void* offset_pthread(void* arg)
{
	uint32_t value[8] = { 0 };

	flash_read(FLASH_USER_DEF, value, 8);

	if (value[0] != MAGIC_NUM)
	{
		value[0] = MAGIC_NUM;

		ctl_offset[0] = 0;
		ctl_offset[1] = 0;
		ctl_offset[2] = 0;
		ctl_offset[3] = 0;

		value[1] = ctl_offset[0];
		value[2] = ctl_offset[1];
		value[3] = ctl_offset[2];
		value[4] = ctl_offset[3];

		flash_erase(FLASH_USER_DEF);
		flash_write(FLASH_USER_DEF, value, 1);
	}

	flash_read(FLASH_USER_DEF, value, 8);
	ctl_offset[0] = value[1];
	ctl_offset[1] = value[2];
	ctl_offset[2] = value[3];
	ctl_offset[3] = value[4];

	while (1)
	{
		int changed = 0;

		if (value[0] != MAGIC_NUM)
		{
			value[0] = MAGIC_NUM;
			changed = 1;
		}
		if (value[1] != ctl_offset[0])
		{
			value[1] = ctl_offset[0];
			changed = 1;
		}
		if (value[2] != ctl_offset[1])
		{
			value[2] = ctl_offset[1];
			changed = 1;
		}
		if (value[3] != ctl_offset[2])
		{
			value[3] = ctl_offset[2];
			changed = 1;
		}
		if (value[4] != ctl_offset[3])
		{
			value[4] = ctl_offset[3];
			changed = 1;
		}

		if (changed)
		{
			flash_erase(FLASH_USER_DEF);
			flash_write(FLASH_USER_DEF, value, 8);
		}
		sleep_ticks(1000);
	}

	return NULL;
}

void* nrf2401_pthread(void* arg)
{
	uint8_t buff_ctl[FIXED_PACKET_LEN] = { 0 };
	buff_ctl[0] = 0xA5;
	buff_ctl[1] = 0x5A;

	NRF24L01_check();
	RF24L01_Init();
	RF24LL01_Write_Hopping_Point(64);
	NRF24L01_Set_Power(POWER_0DBM);
	NRF24L01_Set_Speed(SPEED_250K);

	uint32_t tk = 0;

	while (1)
	{
		ctl_set_offset();

		uint16_t btn = 0x0;
		if (ctl_btn[8] == 1)
		{
			btn |= 1;
		}
		if (ctl_btn[9] == 1)
		{
			btn |= (1 << 1);
		}
		if (ctl_btn[10] == 0)
		{
			btn |= (1 << 2);
		}
		if (ctl_btn[11] == 0)
		{
			btn |= (1 << 3);
		}

		int ind = 2; // HEAD0 HEAD2
		buff_ctl[ind++] = pwm[0] & 0xff;
		buff_ctl[ind++] = (pwm[0] >> 8) & 0xff;

		buff_ctl[ind++] = pwm[1] & 0xff;
		buff_ctl[ind++] = (pwm[1] >> 8) & 0xff;

		buff_ctl[ind++] = pwm[2] & 0xff;
		buff_ctl[ind++] = (pwm[2] >> 8) & 0xff;

		buff_ctl[ind++] = pwm[3] & 0xff;
		buff_ctl[ind++] = (pwm[3] >> 8) & 0xff;

		buff_ctl[ind++] = btn & 0xff;
		buff_ctl[ind++] = (btn >> 8) & 0xff;

		uint16_t crc = calc16crc(buff_ctl, ind); //除CRC之外

		buff_ctl[ind++] = crc & 0xff;
		buff_ctl[ind++] = (crc >> 8) & 0xff;

		if (tx_send == 0)
		{
			RF24L01_Set_Mode(MODE_TX);
			NRF24L01_TxPacket(buff_ctl, FIXED_PACKET_LEN);
			tx_send = 1;
		}

		if (tk % 2 == 0)
		{
			// for (int i = 0; i < ind; i++)
			// {
			// 	printf("%02x ", buff_ctl[i]);
			// }
			// printf("\n");
			
			// printf("%4d %4d %4d %4d %x\n", pwm[0], pwm[1], pwm[2], pwm[3], btn);
			// printf("%4d %4d %4d %4d\n", ctl_offset[0], ctl_offset[1], ctl_offset[2], ctl_offset[3]);
		}

		tk++;
		sleep_ticks(20);
	}
	return NULL;
}

void nrf2401_task(void)
{
	pcb_create(PCB_NRF_PRIO, &nrf2401_pthread, NULL, PCB_NRF_SIZE);
	pcb_create(PCB_OSSAVE_PRIO, &offset_pthread, NULL, PCB_OSSAVE_SIZE);
}


void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
	if (GPIO_Pin == GPIO_PIN_10)
	{
		NRF24L01_Clear_IRQ_Flag(IRQ_ALL);
		tx_send = 0;
	}
}
